@@ -118,7 +118,7 @@ char *newline = "";
 #define sigaltstack sigstack
 #endif
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(__DJGPP__)
 void
 setup_error_handler ()
 {
@@ -133,8 +133,11 @@ setup_error_handler ()
   sigstk.ss_size = SIGSTKSZ;
   sigstk.ss_flags = 0;
 #endif /* ! _AIX */
+
+#ifndef _UNICOS
   if (sigaltstack (&sigstk, 0) < 0)
     perror("sigaltstack");
+#endif
 
   /* Initialize structure for sigaction (called below).  */
   act.sa_handler = cleanup_and_exit;
@@ -168,7 +171,7 @@ setup_error_handler ()
   sigaction (SIGBUS, &act, 0);
   sigaction (SIGFPE, &act, 0);
 }
-#endif /* ! _WIN32 */
+#endif /* ! _WIN32 && ! __DJGPP__ */
 
 main (int argc, char **argv)
 {
@@ -179,7 +182,7 @@ main (int argc, char **argv)
   char *str;
   int base = 10;
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !defined(__DJGPP__)
   setup_error_handler ();
 #endif
 
